require_relative 'core_ext/hash'
require_relative '../lib/structure_validator'

describe StructureValidator do

  let(:hash_to_validate) do
    {
      item: {
        name: 'Foo',
        count: 3,
        data: {
          some: 'value'
        }.to_json
      }
    }
  end

  let(:expected_structure) do
    {
      item: {
        name: 'string',
        count: 'integer',
        data: 'json'
      }
    }
  end

  it "validates the hash" do
    expect(StructureValidator.validate(hash_to_validate, expected_structure)).to be_true
  end

  it "checks required keys" do
    hash_to_validate[:item].delete(:count)
    expect(StructureValidator.validate(hash_to_validate, expected_structure)).to be_false
  end

  it "raises error with an unexpected type" do
    expected_structure.deep_merge!(item: { name: 'foo' })
    expect {
      StructureValidator.validate(hash_to_validate, expected_structure)
    }.to raise_error(StructureValidator::UnsupportedDataType, 'Unexpected type: foo')
  end

  it "checks the key type" do
    hash_to_validate.merge!(item: { count: 'foo' })
    expect(StructureValidator.validate(hash_to_validate, expected_structure)).to be_false
  end

end
